import torch
import torchvision
import torchvision.transforms as transforms


# This file provides easy-to-use functions that create
# train/test data loaders for MNIST, CIFAR-10, and CIFAR-100.

def get_mnist_loaders(batch_size=128, num_workers=2, root="./data"):
    # Standard transform for MNIST: convert to tensor and normalize
    transform = transforms.Compose([transforms.ToTensor(), transforms.Normalize((0.1307,), (0.3081,))])

    # Download & load train / test set
    train_dataset = torchvision.datasets.MNIST(root=root, train=True, download=True, transform=transform)
    test_dataset = torchvision.datasets.MNIST(root=root, train=False, download=True, transform=transform)

    # Build data loaders for training & testing
    train_loader = torch.utils.data.DataLoader(train_dataset, batch_size=batch_size, shuffle=True,
        num_workers=num_workers)
    test_loader = torch.utils.data.DataLoader(test_dataset, batch_size=batch_size, shuffle=False,
        num_workers=num_workers)

    return train_loader, test_loader


def get_cifar10_loaders(batch_size=128, num_workers=2, root="./data"):
    # Typical normalization values for CIFAR-10
    train_transform = transforms.Compose(
        [transforms.ToTensor(), transforms.Normalize((0.4914, 0.4822, 0.4465), (0.2023, 0.1994, 0.2010)), ])
    test_transform = transforms.Compose(
        [transforms.ToTensor(), transforms.Normalize((0.4914, 0.4822, 0.4465), (0.2023, 0.1994, 0.2010)), ])

    train_dataset = torchvision.datasets.CIFAR10(root=root, train=True, download=True, transform=train_transform)
    test_dataset = torchvision.datasets.CIFAR10(root=root, train=False, download=True, transform=test_transform)

    # Create data loaders for CIFAR-10
    train_loader = torch.utils.data.DataLoader(train_dataset, batch_size=batch_size, shuffle=True,
        num_workers=num_workers)
    test_loader = torch.utils.data.DataLoader(test_dataset, batch_size=batch_size, shuffle=False,
        num_workers=num_workers)

    return train_loader, test_loader


def get_cifar100_loaders(batch_size=128, num_workers=2, root="./data"):
    # Normalization stats for CIFAR-100
    train_transform = transforms.Compose(
        [transforms.ToTensor(), transforms.Normalize((0.5071, 0.4867, 0.4408), (0.2675, 0.2565, 0.2761)), ])
    test_transform = transforms.Compose(
        [transforms.ToTensor(), transforms.Normalize((0.5071, 0.4867, 0.4408), (0.2675, 0.2565, 0.2761)), ])

    train_dataset = torchvision.datasets.CIFAR100(root=root, train=True, download=True, transform=train_transform)
    test_dataset = torchvision.datasets.CIFAR100(root=root, train=False, download=True, transform=test_transform)

    # Create data loaders for CIFAR-100
    train_loader = torch.utils.data.DataLoader(train_dataset, batch_size=batch_size, shuffle=True,
        num_workers=num_workers)
    test_loader = torch.utils.data.DataLoader(test_dataset, batch_size=batch_size, shuffle=False,
        num_workers=num_workers)

    return train_loader, test_loader
